<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class RechargePagSeguro extends \Zeedhi\Framework\Controller\Simple {

    protected $rechargeService;
    protected $profileService;
    protected $snackMessage;
    protected $util;
    protected $smtpService;
    protected $balanceService;
    

    public function __construct(\Service\Recharge $rechargeService, \Service\Profile $profileService, \Util\SnackMessage $snackMessage, \Util\Util $util, \Service\SMTP $smtpService, \Service\Balance $balanceService){
        $this->rechargeService = $rechargeService;
        $this->profileService  = $profileService;
        $this->snackMessage    = $snackMessage;
        $this->util            = $util;
        $this->smtpService     = $smtpService;
        $this->balanceService  = $balanceService;
    }

    public function startSession(Request\Filter $request, Response $response){
        $pagseguro_email = 'pagseguro_financeiro@mercadodaboca.com.br';
        $pagseguro_token = 'E6663EFDCDF44AEC9691332AB6F865E6';

        \PagSeguro\Library::initialize();
		\PagSeguro\Library::cmsVersion()->setName("Nome")->setRelease("1.0.0");
		\PagSeguro\Library::moduleVersion()->setName("Nome")->setRelease("1.0.0");
		
		\PagSeguro\Configuration\Configure::setEnvironment('sandbox');//production or sandbox
		\PagSeguro\Configuration\Configure::setAccountCredentials(
			$pagseguro_email,
			$pagseguro_token
		);
		\PagSeguro\Configuration\Configure::setCharset('UTF-8');// UTF-8 or ISO-8859-1
		\PagSeguro\Configuration\Configure::setLog(true, 'log_pagseguro.log');
		
		try {
			$sessionCode = \PagSeguro\Services\Session::create(
				\PagSeguro\Configuration\Configure::getAccountCredentials()
			);
			
            $result['session_id'] = $sessionCode->getResult();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('SessionIdRepo', $result));
        
		} catch (Exception $e) {
			die($e->getMessage());
		}
    }

    public function pagSeguroRecharge(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();

        $valorPedido    = $params[0]['value'];
        $cardHash       = $params[1]['value'];
        $cardToken      = $params[2]['value'];
        $numeroCartao   = $params[3]['value'];
        $CDCLIENTE      = $params[4]['value'];
        $CDCONSUMIDOR   = $params[5]['value'];
        $rechargeValues = $params[6]['value'];
        $CDFILIAL       = $params[7]['value'];
        $appUrl         = $params[8]['value'];
        $logoEmail      = $params[9]['value'];

        $logoEmpresaInfo = array(
            'appUrl'      => $appUrl,
            'logoEmail' => $logoEmail
        );

        $NRTRANSACAO = $this->util->newCode('TRANSACAO', 12);

        // Creating consumers list to send e-mail.
        $consumidores = '';
        $consumidorJaAnotado = array();
        foreach ($rechargeValues as $value){
            if ($consumidores == ''){
                $consumidores = $value['NMCONSUMIDOR'];
                array_push($consumidorJaAnotado, $value);
            }
            else {
                foreach ($consumidorJaAnotado as $consumidor){
                    if ($value['CDCONSUMIDOR'] != $consumidor['CDCONSUMIDOR']){
                        $consumidores = $consumidores . ', ' .$value['NMCONSUMIDOR'];
                    }
                }
            }
        }

        // Call method to insert into table TRANSACAO
        $transacaoParams = array(
            'NRTRANSACAO'  => $NRTRANSACAO,
            'DSTRANSACAO'  => "Transacao iniciada.",
            'CDCLIENTE'    => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'VRMOVEXTCONS' => $valorPedido
        );
        $this->rechargeService->insertTransacao($transacaoParams);

        $controlArray = array();
        foreach ($rechargeValues as $consumer){
            if (!($this->consumerAlreadyRegistered($consumer['CDCONSUMIDOR'], $controlArray))){
                $transactionConsParams = array (
                    'NRTRANSACAO' => $NRTRANSACAO,
                    'CDCLIENTE' => $CDCLIENTE,
                    'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR']
                );
                $this->rechargeService->insertTransacaoCons($transactionConsParams);
            }
            array_push($controlArray, $consumer['CDCONSUMIDOR']);
        }

        $consumerData = $this->profileService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);
        $consumerData = $consumerData[0];

        $pagseguro_email = 'pagseguro_financeiro@mercadodaboca.com.br';
        $pagseguro_token = 'E6663EFDCDF44AEC9691332AB6F865E6';

        \PagSeguro\Library::initialize();
        \PagSeguro\Library::cmsVersion()->setName("Nome")->setRelease("1.0.0");
        \PagSeguro\Library::moduleVersion()->setName("Nome")->setRelease("1.0.0");

        \PagSeguro\Configuration\Configure::setEnvironment('sandbox');//production or sandbox
        \PagSeguro\Configuration\Configure::setAccountCredentials(
                $pagseguro_email,
                $pagseguro_token
        );
        \PagSeguro\Configuration\Configure::setCharset('UTF-8');// UTF-8 or ISO-8859-1
        \PagSeguro\Configuration\Configure::setLog(true, 'log_pagseguro.log');

        try {
            $creditCard = new \PagSeguro\Domains\Requests\DirectPayment\CreditCard();
            $creditCard->setMode('DEFAULT');
            $creditCard->setCurrency("BRL");      
            $creditCard->setSender()->setHash($cardHash);
            $creditCard->setToken($cardToken);
            $creditCard->setReceiverEmail('pagseguro_financeiro@mercadodaboca.com.br');
            $creditCard->setReference($NRTRANSACAO);
            $creditCard->setSender()->setName($consumerData['NMRESPCONS']);
            $creditCard->setSender()->setDocument()->withParameters('CPF', $consumerData['NRCPFRESPCON']);
            $creditCard->setSender()->setPhone()->withParameters(substr($consumerData['NRTELECONS'], 0, 2), substr($consumerData['NRTELECONS'], 2));
            $creditCard->setSender()->setEmail('teste@sandbox.pagseguro.com.br');

            $creditCard->getShipping()->setAddressRequired(false);
            
            $creditCard->setInstallment()->withParameters(1, $valorPedido);

            $creditCard->setBilling()->setAddress()->withParameters(
                $consumerData['DSENDECONS'], //cobranca_endereco
                $consumerData['NRENDECONS'], //cobranca_numero
                $consumerData['NMBAIRRO'], //cobranca_bairro
                str_replace('-', '', $consumerData['NRCEPCONS']), //cobranca_cep
                $consumerData['NMMUNICIPIO'], //cobranca_cidade
                $consumerData['SGESTADO'], //cobranca_estado
                $consumerData['NMPAIS'], //cobranca_pais
                ''    
            );

            $creditCard->setHolder()->setName($consumerData['NMRESPCONS']); // Igual ao cartão de credito
            $creditCard->setHolder()->setPhone()->withParameters(substr($consumerData['NRTELECONS'], 0, 2), substr($consumerData['NRTELECONS'], 2));
            $creditCard->setHolder()->setDocument()->withParameters('CPF', $consumerData['NRCPFRESPCON']);
            
            $creditCard->addItems()->withParameters(
                1,
                'Recarga NoCash',
                1,
                $valorPedido
            );
            
            $transactionResult = $creditCard->register(\PagSeguro\Configuration\Configure::getAccountCredentials());

            if($transactionResult->getStatus() === '3'){
                foreach ($rechargeValues as $family) {
                    $isFirstInsertion = $this->rechargeService->isFirstInsertion($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $currentBalance = $this->rechargeService->getCurrentBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $valueToRecharge = floatval(str_replace(',', '.', $family['VRSALDCONFAM']));
    
                    // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                    $balance = $this->balanceService->checkBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
    
                    $VRSALDCONEXT = $valueToRecharge;
                    //If it's not first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
                    if(!empty($isFirstInsertion)){
                        //Add VRSALDCONEXT to previous balance
                        if(!empty($currentBalance)){
                            $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                        }
                    }
                    $this->rechargeService->insertExtratoCons($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito No Cash (Cartao)', $valueToRecharge, 'C', $VRSALDCONEXT);
    
                    //If there is no record in SALDOCONS, we have to create one with current valueToRecharge
                    if (empty($balance)){
                        // Without any balance, the balance can't be digivolved, so we must INSERT ADDITIONAL BALANCES!
                        $this->balanceService->createBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $valueToRecharge);
                    } else {
                        //If there is balance already, we have to update it with the new value
                        $this->balanceService->updateBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                    }
                }
                $result = true;
    
                // Update TRANSACAO case success.
                $transacaoParams = array(
                    'NRTRANSACAO' => $NRTRANSACAO,
                    'DSTRANSACAO' => 'Transação feita com sucesso.',
                    'NRNIT'       => $transactionResult->getCode()
                );
                $this->rechargeService->updateTransacao($transacaoParams);
    
                // E-mail - The consumerDetails is the same for all the consumers with the same CDCLIENTE, so we don't need to build a foreach to each one.
                $codigoCartao = substr($numeroCartao, 0, 4).' '.substr($numeroCartao, 4, 2).'XX XXXX XX'.substr($numeroCartao, 14);
    
                // Formating values to send e-mail
                $valorPedido = str_replace(".", "", $valorPedido);
                $valorPedido = substr($valorPedido, 0, (strlen($valorPedido)-2)).','.substr($valorPedido, -2, strlen($valorPedido));
                $temp        = $valorPedido.substr($valorPedido, 2);
    
                $nomeDestinatario = $consumerData['NMRESPCONS'];
                $emailDestinatario = $consumerData['DSEMAILCONS'];
    
                // Creating beautiful e-mail template.
                $subject = $this->snackMessage->getMessage('24');
                $sender  = $this->snackMessage->getMessage('14');
                $content =
                    '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
                    '<b>Nome do(s) Aluno(s): </b>'.$consumidores.'<br>'.
                    '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
                    '<p><b>Valor Total da Recarga: </b>R$ '.$valorPedido.'</p>'.
                    '<p><b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y") .'</p>'
                ;
    
                try {
                    // Sends the e-mail!
                    $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
                    $mailSent = true;
                } catch (\Exception $e){
                    $mailSent = false;
                }
    
                /* *** E-MAIL SENT *** */
                $result = true; // Indicates everything was done successfully.
                if ($mailSent) $response->addMessage(new Message('Transação feita com sucesso!'));
                else $response->addMessage(new Message('Transação feita com sucesso!'." ".$this->snackMessage->getMessage('25')));
    
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
            } else{
                throw new \Exception();
            }
            
        } catch (\Exception $e) { 
            // Something went wrong with the payment.
            $message = substr($e->getMessage(),0,255);
            // Update TRANSACAO case failure.
            $transacaoParams = array(
                'NRTRANSACAO' => $NRTRANSACAO,
                'DSTRANSACAO' => $message,
                'NRNIT' => null
            );
            $this->rechargeService->updateTransacao($transacaoParams);
            $response->addMessage(new Message($message));
            $result = false; // Indicates something went wrong.
            $response->addMessage(new Message($e->getMessage()));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result, 'MESSAGE' => 'Não foi possível concluir o pagamento.')));
        }
    }

    public function consumerAlreadyRegistered($consumer, $consumersRegistered){
        foreach ($consumersRegistered as $cons) {
            if($consumer == $cons){
                return true;
            }
        }
        return false;
    }

}