<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class SalesEmail extends \Zeedhi\Framework\Controller\Simple {

    protected $salesEmailService;
    protected $smtpService;

    public function __construct(\Service\SalesEmail $salesEmailService, \Service\SMTP $smtpService){
        $this->salesEmailService = $salesEmailService;
        $this->smtpService = $smtpService;
    }

    public function run() {
        try {
            
            // Logging.
            $this->salesEmailService->writeLog("- Iniciando envio de emails.\n");

            $DTULTEXEC = $this->salesEmailService->getLastRunDate();
            
            $NRLOGEMAILVEND = $this->salesEmailService->newCode('LOGEMAILVENDA', 10);
            
            if($DTULTEXEC !== NULL){
                $this->salesEmailService->saveLogInDataBase($NRLOGEMAILVEND, "Iniciando envio de emails de comprovante de consumo.");

                $processedSales = 0;
                
                // Processing sales
                $this->salesEmailService->writeLog("- Iniciando procesamento de emails de comprovante de consumo.\n");
                $this->salesEmailService->updateLogInDataBase($NRLOGEMAILVEND, "Procesando emails de comprovante de consumo.", 'N');
    
                $salesProd = $this->salesEmailService->getSales($DTULTEXEC);
                $sales = $this->getDistinctSales($salesProd);
    
                $this->salesEmailService->writeLog("- ".count($sales)." vendas encontradas.\n");
                
                $logoEmailInfo['appUrl'] = 'http://heitorcarvalho1.zeedhi.com/workfolder/noCashApp/';
                $logoEmailInfo['logoEmail'] = 'logo_empresa.png';
    
                foreach ($sales as &$sale){
                    $saleDetais = $this->getSaleDetails($sale, $salesProd);
    
                    $nomeResponsavel = $saleDetais[0]['NMRESPCONS'];
                    $nomeAluno = $saleDetais[0]['NMCONSUMIDOR'];
                    $familiaSaldo = $saleDetais[0]['NMFAMILISALD'];
                    $valorPedido = $saleDetais[0]['VRMOVEXTCONS'];
                    $valorNovoSaldo = $saleDetais[0]['VRSALDCONEXT'];
                    $dataPedido = $saleDetais[0]['DTMOVEXTCONS'];
    
                    $DSSALEEMAIL = "<p><b>Nome do Respons&aacute;vel: </b>". $nomeResponsavel ."<br>".
                                   "<b>Nome do Aluno: </b>". $nomeAluno ."<br>".
                                   "<b>Família de saldo: </b>". $familiaSaldo ."<br>".
                                   "<b>Valor Total da Compra: </b>R$ ". 1 * $valorPedido ."<br>".
                                   "<b>Novo saldo: </b>R$ ". 1 * $valorNovoSaldo ."<br>".
                                   "<b>Data da Transa&ccedil;&atilde;o: </b>". $dataPedido ."<br><br>".
                                   "<b>Produtos:</b><br><br>";
                    
                    foreach($salesProd as $prod){
                        if($prod['NRSEQVENDA'] === $sale){
                            $DSSALEEMAIL = $DSSALEEMAIL . $prod['QTPRODVEND'] . "x " . $prod['NMPRODUTO'] . " - R$ " . (($prod['VRUNITVEND'] * $prod['QTPRODVEND']) - $prod['VRDESITVEND']) . "<br>";
                        }
                    }
    
                    $processedSales++;
                    $this->smtpService->sendEmail($saleDetais[0]['NMRESPCONS'], $saleDetais[0]['DSEMAILCONS'], 'No Cash', 'Comprovante de compra com créditos', $DSSALEEMAIL, $logoEmailInfo);
                }
    
                $this->salesEmailService->writeLog("- Envio de emails concluído.\n");
                $this->salesEmailService->writeLog("- ".$processedSales." vendas processadas.\n\n\n");
                $this->salesEmailService->updateLogInDataBase($NRLOGEMAILVEND, 'Envio de emails concluído.', 'S');
            } else{
                $this->salesEmailService->writeLog("- Primeira execucao.\n");
                $this->salesEmailService->saveLogInDataBase($NRLOGEMAILVEND, "Primeira execucao.");
                $this->salesEmailService->updateLogInDataBase($NRLOGEMAILVEND, "fim da primeira execucao.", 'S');
            }
        } catch(\Exception $e){
            $this->salesEmailService->writeLog("- ".$e->getMessage());
            $this->salesEmailService->writeLog("- ".$processedSales." vendas processadas.\n\n\n");
            $this->salesEmailService->updateLogInDataBase($NRLOGEMAILVEND, $e->getMessage(), 'N');
        }
    }

    public function getDistinctSales($salesProd){
        $distinctSales = array();

        foreach($salesProd as $prod){
            $pushToArray = true;

            foreach($distinctSales as $sale){
                if ($prod['NRSEQVENDA'] === $sale){
                    $pushToArray = false;
                }
            }
            
            if($pushToArray){
                array_push($distinctSales, $prod['NRSEQVENDA']);
            }
        }

        return $distinctSales;
    }

    public function getSaleDetails($sale, $salesProd){
        $saleDetais = array();

        foreach($salesProd as $prod){
            if ($prod['NRSEQVENDA'] === $sale){
                array_push($saleDetais, $prod);
            }
        }

        return $saleDetais;
    }

}