<?php

namespace Odhen\API\Service;

class Login {

    protected $entityManager;
    protected $util;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Util\Util $util){
        $this->entityManager = $entityManager;
        $this->util = $util;
    }

    public function validaLoginCaixa($CDFILIAL, $CDCAIXA, $CDVENDEDOR, $CDSENHOPER_DIGITADA, $IDHABCAIXAVENDA, $CDOPERADOR = null){
        try {
            $result = self::validaFilial($CDFILIAL);
            if ($result['error'] == false){
                $result = self::validaCaixa($CDFILIAL, $CDCAIXA);
                if ($result['error'] == false){
                    $result = self::validaModalidadeCaixa($CDFILIAL, $CDCAIXA, $IDHABCAIXAVENDA);
                    if ($result['error'] == false){
                        $result = $this->util->delphiValidaPers($CDFILIAL, $CDCAIXA);
                        if ($result['error'] == false){
                            if ($IDHABCAIXAVENDA == 'TAA'){
                                $result = array(
                                    'error' => false,
                                    'CDOPERADOR' => $CDOPERADOR
                                );
                            } else {
                                $result = self::validaVendedor($CDVENDEDOR, $CDFILIAL, $CDCAIXA);
                            }
                            if ($result['error'] == false) {
                                $CDOPERADOR = $result['CDOPERADOR'];
                                return self::validaOperador($CDOPERADOR, $CDFILIAL, $CDSENHOPER_DIGITADA);
                            }
                        }
                    }
                }
            }
            return $result;
        } catch (\Exception $e) {
            return array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
    }

    private function validaFilial($CDFILIAL) {
        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $filial = $this->entityManager->getConnection()->fetchAll(self::SQL_VALIDA_FILIAL, $params);
        if (!empty($filial)) {
            if ($filial[0]['CDCLIENTE'] != null) {
                $result = array(
                    'error' => false
                );
            } else {
                $result = array(
                    'error' => true,
                    'message' => 'Filial não possui cliente padrão.'
                );
            }
        } else {
            $result = array(
                'error' => true,
                'message' => 'Filial não encontrada.'
            );
        }
        return $result;
    }

    private function validaCaixa($CDFILIAL, $CDCAIXA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA
        );
        $caixa = $this->entityManager->getConnection()->fetchAll(self::SQL_VALIDA_CAIXA, $params);
        if (!empty($caixa)) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'NRCONFTELA' => $caixa[0]['NRCONFTELA']
            );
            $confTela = $this->entityManager->getConnection()->fetchAll(self::SQL_CONFIGURACAO, $params);
            if (!empty($confTela)) {
                $result = array(
                    'error' => false
                );
            } else {
                $result = array(
                    'error' => true,
                    'message' => 'A configuração de tela do caixa não foi encontrada.'
                );
            }
        } else {
            $result = array(
                'error' => true,
                'message' => 'Caixa não encontrado.'
            );
        }
        return $result;
    }

    private function validaModalidadeCaixa($CDFILIAL, $CDCAIXA, $IDHABCAIXAVENDA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'IDHABCAIXAVENDA' => $IDHABCAIXAVENDA
        );
        $modo = $this->entityManager->getConnection()->fetchAll(self::SQL_VALIDA_MODO_HABILITADO, $params);
        if (!empty($modo)) {
            $result = array(
                'error' => false
            );
        } else {
            $result = array(
                'error' => true,
                'message' => 'O caixa não está habilitado para a modalidade selecionada.'
            );
        }
        return $result;
    }

    private function validaVendedor($CDVENDEDOR, $CDFILIAL, $CDCAIXA) {
        $params = array(
            'CDVENDEDOR' => $CDVENDEDOR
        );
        $vendedor = $this->entityManager->getConnection()->fetchAll(self::SQL_VALIDA_VENDEDOR, $params);

        try {

            // SET TRUE TO BYPASS VENDOR VALIDATIONS.
            // SET FALSE TO REQUIRE VENDOR VALIDATIONS.
            $BYPASS = true; // <---------------------

            // VENDOR VALIDATIONS:
            if (empty($vendedor)) throw new \Exception('Vendedor não encontrado.');
            if (($vendedor[0]['CDFILIAL'] == null) && !$BYPASS) throw new \Exception('Vendedor não possui filial associada.');
            if (($vendedor[0]['CDFILIAL'] != $CDFILIAL) && !$BYPASS) throw new \Exception('Vendedor não está associado à filial informada.');
            if (($vendedor[0]['CDCAIXA'] = null) && !$BYPASS) throw new \Exception('Vendedor não possui caixa associado.');
            if (($vendedor[0]['CDCAIXA'] != $CDCAIXA) && !$BYPASS) throw new \Exception('Vendedor não está associado ao caixa informado.');
            if (($vendedor[0]['CDOPERADOR'] == null) && !$BYPASS) throw new \Exception('Vendedor não possui operador associado.');

            // RESULT:
            $result = array(
                'error' => false,
                'CDOPERADOR' => $vendedor[0]['CDOPERADOR']
            );
        } catch (\Exception $e){
            $result = array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
        return $result;
    }

    public function validaOperador($CDOPERADOR, $CDFILIAL, $CDSENHOPER_DIGITADA){
        $params = array(
            'CDOPERADOR' => $CDOPERADOR
        );
        $operador = $this->entityManager->getConnection()->fetchAll(self::SQL_VALIDA_OPERADOR, $params);

        try {

            // OPERATOR VALIDATIONS:
            if (empty($operador)) throw new \Exception('Operador não encontrado.');
            if ($operador[0]['CDSENHOPER'] == null) throw new \Exception('O operador não possui senha cadastrada.');

            $passwordResult = $this->util->delphiValidaSenhaOperador($CDSENHOPER_DIGITADA, $operador[0]['CDSENHOPER']);
            if (!$passwordResult) throw new \Exception('Senha inválida.');

            $params = array(
                'CDOPERADOR' => $CDOPERADOR,
                'CDFILIAL' => $CDFILIAL
            );
            $operFilial = $this->entityManager->getConnection()->fetchAll(self::SQL_OPERADOR_FILIAL, $params);
            if (empty($operFilial)) throw new \Exception('Operador não vinculado à filial informada.');

            // RESULT:
            $result = array(
                'error' => false,
                'CDOPERADOR' => $CDOPERADOR
            );
        } catch (\Exception $e){
            $result = array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
        return $result;
    }

    public function overthrow($P){
        return $this->util->overthrow($P);
    }

    const SQL_VALIDA_FILIAL = "
        SELECT F.CDFILIAL, P.CDCLIENTE
          FROM FILIAL F JOIN PARAVEND P
                          ON F.CDFILIAL = P.CDFILIAL
         WHERE F.CDFILIAL = :CDFILIAL
    ";

    const SQL_CONFIGURACAO = "
        SELECT AM.IDTPBUTTON  , AM.IDTPBUTONAUX, AM.NRBUTTON    , AM.NRBUTTONAUX,
               AM.DSBUTTON    , AM.CDIDENTBUTON, AM.NRCOLORTEXT , AM.NRCOLORBACK,
               AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT, AM.DSIMAGEMBUT
          FROM ITMENUCONFTE AM
         WHERE AM.CDFILIAL   = :CDFILIAL
           AND AM.NRCONFTELA = :NRCONFTELA
    ";

    const SQL_VALIDA_CAIXA = "
        SELECT CDFILIAL, CDCAIXA, NRCONFTELA
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA  = :CDCAIXA
    ";

    const SQL_VALIDA_MODO_HABILITADO = "
        SELECT CDFILIAL, CDCAIXA, IDHABCAIXAVENDA
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND IDHABCAIXAVENDA = :IDHABCAIXAVENDA
    ";

    const SQL_VALIDA_VENDEDOR = "
        SELECT CDVENDEDOR, CDFILIAL, CDCAIXA, CDOPERADOR,
               NMFANVEN
          FROM VENDEDOR
         WHERE CDVENDEDOR = :CDVENDEDOR
    ";

    const SQL_VALIDA_OPERADOR = "
        SELECT CDOPERADOR, CDSENHOPER, NMOPERADOR
         FROM OPERADOR
        WHERE CDOPERADOR = :CDOPERADOR
    ";

    const SQL_OPERADOR_FILIAL = "
        SELECT CDOPERADOR, CDFILIAL
          FROM FILIOPER
         WHERE CDOPERADOR = :CDOPERADOR
           AND CDFILIAL = :CDFILIAL
    ";

}