# Plataforma de pagamento #


## Descrição ##
* Essa interface foi criada para unificar os métodos de pagamento usados pela Teknisa.
* Esse repositório possui uma pasta de exemplos.

  - [MundiPagg](https://mundipagg.com/)
  - [Cielo](https://cielo.com.br/)
  - [Software Express](https://www.softwareexpress.com.br/)
  - [BoletoPHP](http://boletophp.com.br/)

## Como Instalar ##

Adicione no composer.json os códigos abaixo
```json
{
    "repositories": [
        {
            "type": "git",
            "url": "git@192.168.122.56:zeedhi-projects/mundipagg.git"
        }
    ],
    "require": {
        "zeedhi/paymentplatform"        : "0.5.0"
    }
}
```
Rode o comando `composer install` ou `composer update` no mesmo diretório onde se encontra o "composer.json" do seu projeto.


### Pagamento com cartão de crédito não cadastrado no sistema. ###

Plataformas que implementam esse serviço: `MundiPagg`, `Software Express` e `Cielo`.


### Cadastrar cartão e comprar com cartão cadastrado. ###

Plataformas que implementam esse serviço: `MundiPagg` e `Cielo`.


### Cancelando ou consultando uma transação. ###

Plataformas que implementam esse serviço: `MundiPagg`.


### Compra com boleto. ###

Plataformas que implementam esse serviço: `MundiPagg` e `BoletoPHP`.


### MundiPagg: ###

#### Como utilizar:

```php
use Zeedhi\PaymentPlatform\Implementations\MundiPagg\PaymentImpl;
$paymentPlatform = PaymentImpl::factory("Insira sua MerchantKey");
```

#### Bandeiras cartão de crédito:

* Verifique a documentação na pasta "examples".

#### Bancos:

* Verifique a documentação na pasta "examples".

### Software Express: ###

#### Como utilizar:

```php
use Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl;
$paymentPlatform = PaymentImpl::factory("Insira sua MerchantKey", "Insira sua MerchantId", "Insira o ambiente que deseja. (PaymentImpl::SANDBOX ou PaymentImpl::PRODUCTION)");
```

#### Bandeiras cartão de crédito:

* Visa
* Mastercard
* Amex
* Hipercard
* Diners
* Aura 
* Elo.
* Alelo 
* Sodexo.

### Cielo: ###

#### Como utilizar:

```php
use Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl;
$paymentPlatform = PaymentImpl::factory("Insira sua MerchantKey", "Insira sua MerchantId", "Insira o ambiente que deseja. (PaymentImpl::SANDBOX ou PaymentImpl::PRODUCTION)");
```
#### Bandeiras cartão de crédito:

* Verifique a documentação na pasta "examples".  

### Pagar.me: ###

#### Como utilizar:

```php
use Zeedhi\PaymentPlatform\Implementations\PagarMe\PaymentImpl;
$paymentPlatform = PaymentImpl::factory("Insira sua ApiKey");
```
#### Bandeiras cartão de crédito:

* Verifique a documentação na pasta "examples".

### BoletoPHP: ###

#### Como utilizar:

```php
use Zeedhi\PaymentPlatform\Implementations\BoletoPHP\PaymentImpl;
use Zeedhi\PaymentPlatform\BankSlip\StoreBankAccount;
use Zeedhi\PaymentPlatform\BankSlip\Store;

$storeBankInfo = new StoreBankAccount($agency, $agencyDigit, $account, $accountDigit, $ourNumber, $assignorNumber);
$storeAddress = new Address($street, $number, $complement, $neighborhood, $city, $state, $zipCode, $country);
$storeInfo = new Store($fancyName, $storeAddress, $storeBankInfo, $webSite, $cpfCnpj, $businessName, $agreement, $contract);
$paymentPlatform = PaymentImpl::factory($storeInfo);
```

#### Bancos:

* Banco do Brasil
* Santander
* Bradesco
* Itau
* HSBC

## Possíveis Erros ##

* `"Unsupported flag"` : Bandeira de cartão não disponível
* `"Unsupported bank."` : Banco para criação de boletos não disponível
* `"Unsupported Customer Type."` : O tipo de pessoa não está disponível
* `"Unsupported Base URL."` : A URL fornecida não está disponível
* `"Unsupported Factory Flag."` : O tipo de flag não está disponível
* `"HTTP Status: {$statusCode}. API MundiPagg informa o seguinte erro: {$errorMessage}"` : Erro na API MundiPagg
* `"HTTP Status: {$statusCode}. API Cielo informa o seguinte erro: {$errorMessage}"` : Erro na API Cielo
* `"Ocorreu um erro na API Software Express: {$errorMessage}` : Erro na API Software Express
* `"Software Express está indisponível."` : Software Express Indisponível
* `"Conexão com Software Express falhou."` : Erro de conexão com a API Software Express
* `"Ocorreu um erro durante a transação: {$errorMessage}` : Erro no pagamento pelo Software Express
* `"Ocorreu um erro durante o cadastro do cartão: {$errorMessage}` : Erro no cadastro do cartão pelo Software Express