var buildValidateRequiredAnwers = function($scope) {
	window.validateRequiredAnwers = function() {
		var answers = $scope.questionary.answer;
		var questionNotAnswered = $scope.questionary.questions.filter(function(item) {
			return (!answers.hasOwnProperty(item.name) || $scope.isPresent(answers[item.name])) || !item.required;
		});

		var message = questionNotAnswered.map(function(question) {
			return parseInt(question.seqno, 10);
		}).sort().join(" , ");
		for (var q in $scope.questionary.questions) {
			if ($scope.questionary.questions.hasOwnProperty(q)) {
				$scope.questionary.questions[q].error = false;
			}
		}

		if (questionNotAnswered.length > 0) {
			alert("As questões " + message + " são obrigatorias e não foram respondidas!");
			for (q in questionNotAnswered) {
				if (questionNotAnswered.hasOwnProperty(q)) {
					questionNotAnswered[q].error = true;
				}
			}

			return false;
		}

		return true;
	};
};

var buildCheckIfAnswered = function($scope) {
	window.checkIfAnswered = function() {

		var answers = $scope.questionary.answer;
		var questionNotAnswered = $scope.questionary.questions.filter(function(item) {
			return (!answers.hasOwnProperty(item.name) || $scope.isPresent(answers[item.name])) || !item.required;
		});

		//clean
		for (var j in answers) {
			if (answers.hasOwnProperty(j)) {
				answers[j].empty = false;
			}
		}

		//set empty atribute
		for (var i in questionNotAnswered) {
			if (questionNotAnswered.hasOwnProperty(i)) {
				answers[questionNotAnswered[i].name].empty = true;
			}
		}
	};
};

var buildCleanQuestionary = function($scope) {
	window.cleanQuestionary = function() {
		var answers = $scope.questionary.answer;
		for (var i in answers) {
			if (answers[i].type === "short_text" || answers[i].type === "large_text" || answers[i].type === "numeric") {
				answers[i].value = "";
			} else {
				answers[i] = cleanUniqueOrMultipleAnswer(answers[i]);
			}
		}
		checkIfAnswered();
	};
};

var QuestionaryController = function($scope) {
	var UNIQUE = 'unique';
	var MULTIPLE = 'multiple';
	var dependencyQuestions = [UNIQUE, MULTIPLE];

	$scope.$watch('widget.dataSource.data', function() {
		if ($scope.widget.dataSource.data) {
			$scope.questionary = $scope.widget.dataSource.data[0];
		}
		if (!$scope.questionary) {
			$scope.questionary = {};
		}
		if (!$scope.questionary.answer) {
			$scope.questionary.answer = {};
		}
		if ($scope.questionary.questions) {
			$scope.questionary.questions.map(function(question) {
				if (!$scope.questionary.answer[question.name]) {
					$scope.questionary.answer[question.name] = {};
				}
			});
		}
	});
	$scope.showOtherOption = function(question, questionary) {
		questionary.answer[question.name].other_value = true;
	};
	$scope.hideOtherOption = function(question) {
		question.other_value = false;
		var answers = $scope.questionary.answer;
		answers[question.name].other_text = "";
		answers[question.name].other_value = false;
	};

	$scope.isPresent = function(answer) {
		return !(!!answer.value || //answer type unique or multiple choice
			$scope.otherChoiceIsPresent(answer) || //other option is checked
			$scope.textIsPresent(answer)); //answer type text
	};

	$scope.checkIfClean = function(answer, clean) {
		if (clean) {
			answer.other_text = "";
			answer.other_value = false;
		}
	};

	$scope.textIsPresent = function(answer) {
		if (!!answer.value) {
			return (typeof(answer.value) === "string" ? answer.value.trim().length > 0 : false);
		}
	};

	$scope.otherChoiceIsPresent = function(answer) {
		//check by presence of optional text option
		if (!!answer.other_value) {
			return (!!answer.other_text) && (answer.other_text.trim().length > 0);
		}
		//check by presence of usual options
		if (!!answer.options) {
			for (var i in answer.options) {
				if (answer.options[i] === true) {
					return true;
				}
			}
		}
		return false;
	};

	var isDependencyValid = function(question, questionary) {
		var questionFiltered = questionary.questions.filter(function(q) {
			return q.name === question.dependency.question;
		})[0];
		return questionFiltered && dependencyQuestions.indexOf(questionFiltered.type) !== -1;
	};

	var dependencyFlow = {
		'unique': function(question, questionary) {
			return questionary.answer[question.dependency.question].value === question.dependency.value;
		},
		'multiple': function(question, questionary, dependencyQuestion) {
			var result = false;
			if (typeof(question.dependency.value) === "string") {
				question.dependency.value = question.dependency.value.split(',');
			}
			if (Array.isArray(question.dependency.value)) {
				var answers = questionary.answer[question.dependency.question].options;
				var options;
				if (question.dependency.value.length > 0) {
					options = question.dependency.value;
				} else {
					options = dependencyQuestion.options.map(function(option) {
						return option.id;
					});
				}
				result = options.every(function(item) {
					return !!answers[item];
				});
			}
			return result;
		}
	};

	var checkAnswer = function(question, questionary) {
		var result = false;

		var dependencyQuestion = questionary.questions.filter(function(q) {
			return q.name === question.dependency.question;
		})[0];

		if (dependencyQuestion) {
			result = dependencyFlow[dependencyQuestion.type](question, questionary, dependencyQuestion);
			if (!result) {
				questionary.answer[question.name].type = question.type;
				cleanUniqueOrMultipleAnswer(questionary.answer[question.name]);
			}
		}
		return result;
	};

	$scope.checkDependency = function(question, questionary) {
		return Util.isUndefinedOrNull(question.dependency) || question.dependency === false || !isDependencyValid(question, questionary) || checkAnswer(
			question, questionary);
	};

	var cleanUniqueOrMultipleAnswer = function(answer) {
		if (answer.type === UNIQUE) {
			answer.other_value = false;
			answer.other_text = "";
			answer.value = null;
		} else if (answer.type === MULTIPLE) {
			answer.other_value = false;
			answer.other_text = "";
			answer.options = [];
		}
		return answer;
	};

	buildValidateRequiredAnwers($scope);

	buildCheckIfAnswered($scope);

	buildCleanQuestionary($scope);

};